@@ -98,6 +98,8 @@ pub struct Runtime {
     pub(crate) last_input_at: std::time::Instant,
     /// Whether the idle shimmer effect is currently active
     pub(crate) idle_shimmer_active: bool,
+    /// Landing page animation state (when showing landing page)
+    pub(crate) landing_state: Option<crate::landing::LandingState>,
 }
 
 impl Default for Runtime {
@@ -234,6 +236,7 @@ impl Runtime {
             plugins: loaded_plugins,
             last_input_at: std::time::Instant::now(),
             idle_shimmer_active: false,
+            landing_state: None,
         };
 
         // Make keymap and command registry accessible to plugins (e.g., which-key)
@@ -438,6 +441,28 @@ impl Runtime {
                 });
         }
 
+        // Enable keyboard enhancement protocol if supported
+        // This allows terminals (kitty, WezTerm, foot) to disambiguate Ctrl+I from Tab
+        if reovim_sys::terminal::supports_keyboard_enhancement().unwrap_or(false) {
+            use reovim_sys::{
+                ExecutableCommand,
+                event::{KeyboardEnhancementFlags, PushKeyboardEnhancementFlags},
+            };
+
+            match std::io::stdout().execute(PushKeyboardEnhancementFlags(
+                KeyboardEnhancementFlags::DISAMBIGUATE_ESCAPE_CODES,
+            )) {
+                Ok(_) => {
+                    tracing::info!("Keyboard enhancement protocol enabled");
+                }
+                Err(e) => {
+                    tracing::warn!("Failed to enable keyboard enhancement: {}", e);
+                }
+            }
+        } else {
+            tracing::debug!("Keyboard enhancement not supported by terminal");
+        }
+
         runtime
     }
 
@@ -478,9 +503,13 @@ impl Runtime {
                 buf.begin_batch();
             }
         } else if was_insert && !is_insert {
-            // Leaving insert mode: flush batch
+            // Leaving insert mode: flush batch and record jump position
             if let Some(buf) = self.buffers.get_mut(&self.active_buffer_id) {
                 buf.flush_batch();
+                // Record current position as a jump point when leaving insert mode
+                // This allows navigating back to where editing was completed
+                // Use push_current() which doesn't truncate (preserves jump history)
+                self.jump_list.push_current(self.active_buffer_id, buf.cur);
             }
         }
 
@@ -818,6 +847,7 @@ impl Runtime {
             debug!(id, path, "open_file: created new buffer");
             self.active_buffer_id = id;
             self.showing_landing_page = false;
+            self.landing_state = None;
         } else {
             debug!(path, "open_file: failed to create buffer");
         }
@@ -1161,9 +1191,13 @@ impl super::RuntimeContext for Runtime {
                 buf.begin_batch();
             }
         } else if was_insert && !is_insert {
-            // Leaving insert mode: flush batch
+            // Leaving insert mode: flush batch and record jump position
             if let Some(buf) = self.buffers.get_mut(&self.active_buffer_id) {
                 buf.flush_batch();
+                // Record current position as a jump point when leaving insert mode
+                // This allows navigating back to where editing was completed
+                // Use push_current() which doesn't truncate (preserves jump history)
+                self.jump_list.push_current(self.active_buffer_id, buf.cur);
             }
         }
 
@@ -1175,3 +1209,12 @@ impl super::RuntimeContext for Runtime {
         (self.screen.width(), self.screen.height())
     }
 }
+
+impl Drop for Runtime {
+    fn drop(&mut self) {
+        // Clean up keyboard enhancement protocol
+        use reovim_sys::{ExecutableCommand, event::PopKeyboardEnhancementFlags};
+
+        let _ = std::io::stdout().execute(PopKeyboardEnhancementFlags);
+    }
+}
